@@ -178,7 +178,6 @@ visualize_keywords <- function(docs, keywords, prune = TRUE, label_size = 3.2)
     dplyr::rename(Word = .data$text_split) %>%
     dplyr::group_by(.data$Word) %>%
     dplyr::summarize(WordCount = dplyr::n()) %>%
-    dplyr::ungroup(.data) %>%
     dplyr::mutate(`Proportion(%)` = round(.data$WordCount / totalwords * 100, 3)) %>%
     dplyr::arrange(dplyr::desc(.data$WordCount)) %>%
     dplyr::mutate(Ranking = 1:(dplyr::n())) -> data
@@ -427,6 +426,7 @@ keyATM_fit <- function(docs, model, no_keyword_topics,
                     model = abb_model_name(model),
                     keywords = keywords_id, keywords_raw = keywords_raw,
                     no_keyword_topics = no_keyword_topics,
+                    keyword_k = length(keywords_raw),
                     vocab = info$wd_names,
                     model_settings = model_settings,
                     priors = priors,
