@@ -1,9 +1,7 @@
-#' \code{keyATM_fit()} calls \code{keyATM_output()}
-#' 
+#' Create an output object
 #' 
 #' @keywords internal
 #' @import magrittr
-#'
 keyATM_output <- function(model)
 {
   message("Creating an output object. It may take time...")
@@ -28,7 +26,7 @@ keyATM_output <- function(model)
     info$tnames <- paste0("Topic_", 1:model$no_keyword_topics)
   } else {
     # Keywords only
-    info$tnames <- c(paste0("", 1:length(model$keywords)))
+    info$tnames <- c(names(model$keywords_raw))
   }
 
   # theta (document-topic distribution)
@@ -53,6 +51,7 @@ keyATM_output <- function(model)
   # alpha_iter
   if (model$model %in% c("hmm", "ldahmm")) {
     values_iter$alpha_iter <- keyATM_output_alpha_iter_hmm(model, info)
+    values_iter$time_index <- model$model_settings$time_index
   }
 
   if ((model$model %in% c("base", "lda", "label"))) {
@@ -142,7 +141,6 @@ keyATM_output_pi <- function(model_Z, model_S, prior)
   temp %>%
     dplyr::mutate(Proportion = p * 100) %>%
     dplyr::select(-.data$sums) -> pi_estimated
-
   return(pi_estimated)
 }
 
@@ -196,7 +194,6 @@ keyATM_output_theta <- function(model, info)
 
   theta <- as.matrix(theta)
   colnames(theta) <- info$tnames # label seeded topics
-
   return(theta)
 }
 
@@ -222,7 +219,6 @@ keyATM_output_phi <- function(model, info)
     obj <- keyATM_output_phi_calc_lda(all_words, all_topics, 
                                       model$vocab, model$priors$beta, info$tnames)
   }
-  
   return(obj)
 }
 
@@ -349,7 +345,6 @@ keyATM_output_phi_calc_key <- function(all_words, all_topics, all_s, pi_estimate
       phi <- phi_ + prior  # add scalar (don't use label or matrix)
       phi <- phi / Matrix::rowSums(phi)
     }
-
     return(phi)
   }
 
@@ -394,7 +389,6 @@ keyATM_output_phi_calc_key <- function(all_words, all_topics, all_s, pi_estimate
     # This can happen in `by_strata_TopicWord`
     # Do nothing
   }
-  
   return(list(phi = phi, topic_counts = topic_counts, word_counts = word_counts))
 }
 
@@ -431,7 +425,6 @@ keyATM_output_phi_calc_lda <- function(all_words, all_topics, vocab, priors, tna
 
   phi <- phi / Matrix::rowSums(phi)
   rownames(phi) <- tnames
-
   return(list(phi = phi, topic_counts = topic_counts, word_counts = word_counts))
 }
 
@@ -569,47 +562,31 @@ print.keyATM_output <- function(x, ...)
 #' @export
 summary.keyATM_output <- function(object, ...)
 {
-  cat(
-      paste0(
-             "keyATM_output object for the ",
-             object$model,
-             " model. ",
-             "\n"
-            )
-     )
+  print(object) 
 }
 
 
 #' Save a keyATM_output object
 #'
-#' @param x a keyATM_output object (see \code{keyATM()})
-#' @param file a character
-#'
+#' @param x a keyATM_output object (see [keyATM()])
+#' @param file file name to create on disk
+#' @seealso [keyATM()], [weightedLDA()], [keyATMvb()]
 #' @export
 save.keyATM_output <- function(x, file = stop("'file' must be specified"))
 {
   saveRDS(x, file = file)
 }
 
-#' Show a diagnosis plot of log-likelihood and perplexity
-#'
-#' @noRd
-#' @export
-plot.keyATM_output <- function(x, ...)
-{
-  print(plot_modelfit(x, ...))
-}
-
 
 #' Show the top words for each topic
 #'
-#' If \code{show_keyword} is true then words in their seeded categories
-#' are suffixed with a check mark. Words from another seeded category
+#' If \code{show_keyword} is \code{TRUE} then words in their keyword topics
+#' are suffixed with a check mark. Words from another keyword topic
 #' are labeled with the name of that category.
 #'
-#' @param x the output (see \code{keyATM()} and \code{by_strata_TopicWord()})
-#' @param n integer. The number terms to visualize. Default is NULL, which shows all terms.
-#' @param measure character. The way to sort the terms: 'probability' (default) or 'lift'.
+#' @param x the output (see [keyATM()] and [by_strata_TopicWord()])
+#' @param n integer. The number terms to visualize. Default is \code{10}.
+#' @param measure character. The way to sort the terms: \code{probability} (default) or \code{lift}.
 #' @param show_keyword logical. If \code{TRUE}, mark keywords. Default is \code{TRUE}.
 #'
 #' @return An n x k table of the top n words in each topic
@@ -626,7 +603,6 @@ top_words <- function(x, n = 10, measure = c("probability", "lift"),
 top_words.strata_topicword <- function(x, n = 10, measure = c("probability", "lift"),
                                   show_keyword = TRUE)
 {
-
   measure <- match.arg(measure)
   top_words <- lapply(x$phi,  # list of phis
                       function(obj) {
@@ -636,7 +612,6 @@ top_words.strata_topicword <- function(x, n = 10, measure = c("probability", "li
                          word_counts = obj$word_counts, keywords_raw = x$keywords_raw
                        )
                       })
-
   return(top_words)
 }
 
@@ -712,8 +687,8 @@ top_words_calc <- function(n, measure, show_keyword,
 
 #' Show the top topics for each document
 #'
-#' @param x the output from a keyATM model (see \code{keyATM()})
-#' @param n integer. The number of topics to show. Default is 2.
+#' @param x the output from a keyATM model (see [keyATM()])
+#' @param n integer. The number of topics to show. Default is \code{2}.
 #'
 #' @return An n x k table of the top n topics in each document
 #' @import magrittr
@@ -739,8 +714,8 @@ top_topics <- function(x, n = 2)
 
 #' Show the top documents for each topic
 #'
-#' @param x the output from a keyATM model (see \code{keyATM_output()})
-#' @param n How many documents to show. Default: 10
+#' @param x the output from a keyATM model (see [keyATM()])
+#' @param n How many documents to show. Default is \code{10}.
 #'
 #' @return An n x k table of the top n documents for each topic, each number is a document index
 #' @import magrittr
@@ -755,16 +730,15 @@ top_docs <- function(x, n = 10)
     order(xcol, decreasing = TRUE)[1:n]
   }
   
-  res <- apply(x$theta, 2, measuref) %>%
-          tibble::as_tibble()
+  res <- apply(x$theta, 2, measuref) %>% as.data.frame()
   return(res) 
 }
 
 
 #' Estimate subsetted topic-word distribution
 #'
-#' @param x the output from a keyATM model (see \code{keyATM()})
-#' @param keyATM_docs an object generated by \code{keyATM_read()} (see \code{keyATM_read()})
+#' @param x the output from a keyATM model (see [keyATM()])
+#' @param keyATM_docs an object generated by [keyATM_read()]
 #' @param by a vector whose length is the number of documents
 #'
 #' @return strata_topicword object (a list)
@@ -772,20 +746,16 @@ top_docs <- function(x, n = 10)
 #' @export
 by_strata_TopicWord <- function(x, keyATM_docs, by)
 {
-
   # Check inputs
   if (!is.vector(by)) {
     stop("`by` should be a vector.") 
   }
-
   if (!"Z" %in% names(x$kept_values)) {
     stop("`Z` and `S` should be in the output. Please check `keep` option in `keyATM()`.") 
   }
-
   if (!"S" %in% names(x$kept_values)) {
     stop("`Z` and `S` should be in the output. Please check `keep` option in `keyATM()`.") 
   }
-
   if (length(keyATM_docs) != length(by)) {
     stop("The length of `by` should be the same as the length of documents.") 
   }
@@ -806,11 +776,10 @@ by_strata_TopicWord <- function(x, keyATM_docs, by)
                                                    x$kept_values$S[doc_index],
                                                    x$priors$gamma)
                   vocab <- sort(unique(all_words))
-
                   phi_obj <- keyATM_output_phi_calc_key(all_words, all_topics, all_s, pi_estimated,
                                                         x$keywords_raw,
                                                         vocab, x$priors, tnames, model = x)
-                } 
+                }
                )
   names(obj) <- unique_val
 
@@ -821,28 +790,56 @@ by_strata_TopicWord <- function(x, keyATM_docs, by)
 }
 
 
+#' Show covariates information
+#'
+#' @param x the output from the covariate keyATM model (see [keyATM()])
+#' @export
+covariates_info <- function(x) {
+  if (x$model != "covariates" | !("keyATM_output" %in% class(x))) {
+    stop("This is not an output of covariate model")
+  }
+  cat(paste0("Colnames: ", paste(colnames(x$kept_values$model_settings$covariates_data_use), collapse = ", "),
+             "\nStandardized: ", as.character(as.logical(x$kept_values$model_settings$standardize)),
+             "\nFormula: ", paste(as.character(x$kept_values$model_settings$covariates_formula), collapse = " "), "\n\nPreview:\n"))
+  print(utils::head(x$kept_values$model_settings$covariates_data_use))
+}
+
+
+#' Return covariates used in the iteration
+#'
+#' @param x the output from the covariate keyATM model (see [keyATM()])
+#' @export
+covariates_get <- function(x) {
+  if (x$model != "covariates" | !("keyATM_output" %in% class(x))) {
+    stop("This is not an output of covariate model")
+  }
+  return(x$kept_values$model_settings$covariates_data_use) 
+}
+
+
 #' Estimate document-topic distribution by strata (for covariate models)
 #'
-#' @param x the output from a keyATM model (see \code{keyATM()})
-#' @param by_name character. The name of the variable to use.
-#' @param by_values numeric. The values of the variable specified in `by_name`
+#' @param x the output from the covariate keyATM model (see [keyATM()])
+#' @param by_var character. The name of the variable to use.
+#' @param labels character. The labels for the values specified in `by_var` (ascending order).
+#' @param by_values numeric. Specific values for `by_var`, ordered from small to large. If it is not specified, all values in `by_var` will be used.
 #' @param burn_in integer. Burn-in period. If not specified, it is the half of samples. Default is \code{NULL}.
 #' @param parallel logical. If \code{TRUE}, parallelization for speeding up. Default is \code{TRUE}.
 #' @param mc.cores integer. The number of cores to use. Default is \code{NULL}.
-#' @param posterior_mean logical. If \code{TRUE}, the quantity of interest to estimate is the posterior mean. Default is \code{FALSE}.
+#' @param posterior_mean logical. If \code{TRUE}, the quantity of interest to estimate is the posterior mean. Default is \code{TRUE}.
 #'
 #' @return strata_topicword object (a list)
 #' @import magrittr
 #' @export
-by_strata_DocTopic <- function(x, by_name, by_values, burn_in = NULL,
-                               parallel = TRUE, mc.cores = NULL, posterior_mean = FALSE)
+by_strata_DocTopic <- function(x, by_var, labels, by_values = NULL, burn_in = NULL,
+                               parallel = TRUE, mc.cores = NULL, posterior_mean = TRUE)
 {
   # Check inputs
-  variables <- colnames(x$kept_values$model_settings$covariates_data)
-  if (!by_name %in% variables)
-    stop(paste0(by_name, " is not in the set of covariates in keyATM model. ",
+  variables <- colnames(x$kept_values$model_settings$covariates_data_use)
+  if (!by_var %in% variables)
+    stop(paste0(by_var, " is not in the set of covariates in keyATM model. Check with `covariates_info()`.",
                 "Covariates provided are: ", 
-                paste(colnames(x$kept_values$model_settings$covariates_data), collapse=" , ")))
+                paste(colnames(x$kept_values$model_settings$covariates_data_use), collapse=" , ")))
 
   if (is.null(burn_in)) {
     burn_in <- floor(max(x$model_fit$Iteration) / 2) 
@@ -851,7 +848,7 @@ by_strata_DocTopic <- function(x, by_name, by_values, burn_in = NULL,
   # Get info for parallelization
   if (parallel) {
     if (is.null(mc.cores)) {
-      num_core <- parallel::detectCores(all.tests = FALSE, logical = TRUE) - 2L
+      num_core <- max(1, parallel::detectCores(all.tests = FALSE, logical = TRUE) - 2L)
     } else {
       num_core <- mc.cores 
     }
@@ -859,7 +856,6 @@ by_strata_DocTopic <- function(x, by_name, by_values, burn_in = NULL,
     num_core <- 1L
   }
 
-  
   # Info
   used_iter <- x$values_iter$used_iter
   used_iter <- used_iter[used_iter > burn_in]
@@ -867,12 +863,19 @@ by_strata_DocTopic <- function(x, by_name, by_values, burn_in = NULL,
   tnames <- rownames(x$phi)
   Lambda_iter <- x$values_iter$Lambda_iter_rescaled
 
+  if (is.null(by_values)) {
+    by_values <- sort(unique(x$kept_values$model_settings$covariates_data_use[, by_var]))
+  }
+  if (length(by_values) != length(labels)) {
+    stop("Length mismatches. Please check `labels`.")
+  }
+
   if (posterior_mean) {
     res <- lapply(1:length(by_values),
                   function(i) {
                     value <- by_values[i]
                     new_data <- x$kept_values$model_settings$covariates_data_use
-                    new_data[, by_name] <- value
+                    new_data[, by_var] <- value
 
                     # Draw theta
                     obj <- do.call(dplyr::bind_rows,
@@ -882,7 +885,6 @@ by_strata_DocTopic <- function(x, by_name, by_values, burn_in = NULL,
                                                                        new_data,
                                                                        Lambda_iter[[use_index[s]]]
                                                                      ))
-
                                                         rowsum <- Matrix::rowSums(Alpha)
                                                         thetas <- Alpha / rowsum
                                                         thetas <- as.data.frame(thetas)
@@ -893,7 +895,6 @@ by_strata_DocTopic <- function(x, by_name, by_values, burn_in = NULL,
                                                       mc.cores = num_core
                                                      )
                                   )
-
                   })  
   } else {
     set.seed(x$options$seed)
@@ -901,7 +902,7 @@ by_strata_DocTopic <- function(x, by_name, by_values, burn_in = NULL,
                   function(i) {
                     value <- by_values[i] 
                     new_data <- x$kept_values$model_settings$covariates_data_use
-                    new_data[, by_name] <- value
+                    new_data[, by_var] <- value
 
                     # Draw theta
                     obj <- do.call(dplyr::bind_rows,
@@ -911,7 +912,6 @@ by_strata_DocTopic <- function(x, by_name, by_values, burn_in = NULL,
                                                                        new_data,
                                                                        Lambda_iter[[use_index[s]]]
                                                                      ))
-                                                       
                                                         thetas <- t(apply(Alpha, 1, rdirichlet))
                                                         thetas <- Matrix::colMeans(thetas)
                                                         thetas <- t(as.data.frame(thetas))
@@ -923,14 +923,11 @@ by_strata_DocTopic <- function(x, by_name, by_values, burn_in = NULL,
                                                       mc.cores = num_core, mc.set.seed = FALSE
                                                      )
                                   )
-
                   })
   }
   names(res) <- by_values
-
-  obj <- list(theta = tibble::as_tibble(res), by_values = by_values, by_name = by_name)
+  obj <- list(theta = tibble::as_tibble(res), by_values = by_values, by_var = by_var, labels = labels)
   class(obj) <- c("strata_doctopic", class(obj)) 
-
   return(obj)
 }
 
@@ -940,21 +937,19 @@ by_strata_DocTopic <- function(x, by_name, by_values, burn_in = NULL,
 #' @export
 summary.strata_doctopic <- function(object, quantile_vec = c(0.05, 0.5, 0.95), ...)
 {
-  x <- object
-  tables <- lapply(1:length(x$by_values),
+  tables <- lapply(1:length(object$by_values),
                   function(index) {
-                     theta <- x$theta[[index]]
-                     theta_ <- theta[, 1:(ncol(theta)-2)]
+                     theta <- object$theta[[index]]
+                     theta_ <- theta[, 1:(ncol(theta)-1)]
                      q <- as.data.frame(apply(theta_, 2, stats::quantile, quantile_vec))
                      q$Percentile <- c("Lower", "Point", "Upper")
                      q %>% 
                        tidyr::gather(key = "Topic", value = "Value", -"Percentile") %>%
                        tidyr::spread(key = "Percentile", value = "Value") %>%
                        dplyr::mutate(TopicId = 1:(dplyr::n()),
-                                     by = as.character(x$by_values[index])) %>%
+                                     val = object$by_values[index],
+                                     label = object$labels[index]) %>%
                        tibble::as_tibble() -> temp
                   })
-
   return(tables)
 }
-
